using System;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.Xml;

namespace DataBinding {
  
  public class AddressCollection : CollectionBase, ITypedList, IBindingList
  {
    PropertyDescriptorCollection _pdc=new PropertyDescriptorCollection(null);
    
    ListChangedEventArgs resetEvent = new ListChangedEventArgs(ListChangedType.Reset, -1);
    ListChangedEventHandler listChangeHandler;

    bool _isSorted=false;
    ListSortDirection _sortDir=ListSortDirection.Ascending;
    PropertyDescriptor _sortProperty;

    public AddressCollection() {
    
      XmlDocument doc=new XmlDocument();
      doc.Load(@"..\..\..\address.xml");
      XmlNodeList nodes=doc.SelectNodes("addressBook/addressEntry");
      foreach(XmlNode node in nodes)  {
        Address tmpAdd=new Address();
        tmpAdd.Name=node.SelectSingleNode("name").InnerText;
        tmpAdd.AddressLine1=node.SelectSingleNode("address1").InnerText;
        tmpAdd.AddressLine2=node.SelectSingleNode("address2").InnerText;
        tmpAdd.City=node.SelectSingleNode("city").InnerText;
        tmpAdd.State=node.SelectSingleNode("st").InnerText;
        tmpAdd.Zip=node.SelectSingleNode("zip").InnerText;
        tmpAdd.PhoneNumbers=new PhoneCollection(node.SelectNodes("phone"));
        //this.List.Add(tmpAdd);
        this.Add(tmpAdd);
      }
      
      _pdc.Add(new AddressPropertyDescriptor("Name", "DataBinding.Address"));
      _pdc.Add(new AddressPropertyDescriptor("AddressLine1", "DataBinding.Address"));
      _pdc.Add(new AddressPropertyDescriptor("AddressLine2", "DataBinding.Address"));
      _pdc.Add(new AddressPropertyDescriptor("City", "DataBinding.Address"));
      _pdc.Add(new AddressPropertyDescriptor("State", "DataBinding.Address"));
      _pdc.Add(new AddressPropertyDescriptor("Zip", "DataBinding.Address"));
      _pdc.Add(new AddressPropertyDescriptor("PhoneNumbers", "DataBinding.Address"));
      
      OnListChanged(resetEvent);
    }

    //ITypedList implementation
    public PropertyDescriptorCollection GetItemProperties(PropertyDescriptor[] arrDesctiptors)  { 
      return _pdc;
    }

    public string GetListName(PropertyDescriptor[] arrDescriptors) {
      return "List";
    }


    public PropertyDescriptor GetProperty(string propertyName)  {
      return _pdc.Find(propertyName,true);
    }

    public int Add(Address value)  {
      return this.List.Add(value);
    }

    public bool Contains(object value)  {
      return this.List.Contains((Address)value);
    }


    public void Insert(int Index, object value) {
      this.List.Insert(Index,(Address)value);
    }

    public void Remove(object value) {
      this.List.Remove((Address)value);
    }

        

    protected override void OnClear() {
      foreach (Address c in List) {
        c.Parent = null;
      }
    }

    protected override void OnClearComplete() {
      OnListChanged(resetEvent);
    }

    protected override void OnInsertComplete(int index, object value) {
      ((Address)value).Parent = this;
      OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, index));
    }

    protected override void OnRemoveComplete(int index, object value) {
      ((Address)value).Parent = this;
      OnListChanged(new ListChangedEventArgs(ListChangedType.ItemDeleted, index));
    }

    protected override void OnSetComplete(int index, object oldValue, object newValue) {
      if (oldValue != newValue) {

        Address oldcust = (Address)oldValue;
        Address newcust = (Address)newValue;
                
        oldcust.Parent = null;
        newcust.Parent = this;
                
            
        OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, index));
      }
    }
        
    // Called by Customer when it changes.
    internal void AddressChanged(Address changedAddress) {
         
      int index = List.IndexOf(changedAddress);
            
      OnListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, index));
    }



    //IBindingList

    protected virtual void OnListChanged(ListChangedEventArgs ev) {
      if (listChangeHandler != null) {
        listChangeHandler(this, ev);
      }
    }
    
    
    public Address AddNew() {
      //forwrd to the IBindingList implementation
      return (Address)((IBindingList)this).AddNew(); 
    }

    bool IBindingList.AllowEdit { 
      get { return true ; }
    }

    bool IBindingList.AllowNew { 
      get { return true ; }
    }

    bool IBindingList.AllowRemove { 
      get { return true ; }
    }

    bool IBindingList.SupportsChangeNotification { 
      get { return true ; }
    }
        
    bool IBindingList.SupportsSearching { 
      get { return false ; }
    }

    bool IBindingList.SupportsSorting { 
      get { return true ; }
    }
    public event ListChangedEventHandler ListChanged {
      add {
        listChangeHandler += value;
      }
      remove {
        listChangeHandler -= value;
      }
    }

    // Methods.

    object IBindingList.AddNew() {
      Address newAddress=new Address();
      newAddress.Parent=this;
      int newPos=List.Add(newAddress);
      OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded,newPos));
      return newAddress;
    }


    // Unsupported properties.
    public bool IsSorted { 
      get { return _isSorted; }
    }

    public ListSortDirection SortDirection { 
      get { return _sortDir; }
    }


    PropertyDescriptor IBindingList.SortProperty { 
      get { return _sortProperty; }
    }


    // Unsupported Methods.
    void IBindingList.AddIndex(PropertyDescriptor property) {
      throw new NotSupportedException(); 
    }

    public void ApplySort(PropertyDescriptor property, ListSortDirection direction) {
      _sortProperty=property;
      _sortDir=direction;
      this.InnerList.Sort(new AddressComparer(_sortProperty)); 
      if(_sortDir==ListSortDirection.Descending)
        this.InnerList.Reverse();

      _isSorted=true;
    }

    int IBindingList.Find(PropertyDescriptor property, object key) {
      throw new NotSupportedException(); 
    }

    void IBindingList.RemoveIndex(PropertyDescriptor property) {
      throw new NotSupportedException(); 
    }

    void IBindingList.RemoveSort() {
      throw new NotSupportedException(); 
    }

  }
}
