using System;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.Xml;

namespace DataBinding {
  /// <summary>
  /// Summary description for AlphaCollection.
  /// </summary>
  public class AddressCollection : System.Collections.ICollection, System.Collections.IList  
  {
    
    ArrayList _objList=new ArrayList();

    public AddressCollection() {
    
      XmlDocument doc=new XmlDocument();
      doc.Load(@"..\..\..\address.xml");
      XmlNodeList nodes=doc.SelectNodes("addressBook/addressEntry");
      foreach(XmlNode node in nodes)  {
        Address tmpAdd=new Address();
        tmpAdd.Name=node.SelectSingleNode("name").InnerText;
        tmpAdd.AddressLine1=node.SelectSingleNode("address1").InnerText;
        tmpAdd.AddressLine2=node.SelectSingleNode("address2").InnerText;
        tmpAdd.City=node.SelectSingleNode("city").InnerText;
        tmpAdd.State=node.SelectSingleNode("st").InnerText;
        tmpAdd.Zip=node.SelectSingleNode("zip").InnerText;
        tmpAdd.PhoneNumbers=new PhoneCollection(node.SelectNodes("phone"));
        this.Add(tmpAdd);
      }

    }
    
    //ICollection
    virtual public int Count  {
      get {return _objList.Count;}
    }

    virtual public bool IsSynchronized  {
      get {return false;}
    }

    virtual public object SyncRoot  {
      get {return this;}
    }

    virtual public void CopyTo(Array targetArray, int index)  {
      _objList.CopyTo(targetArray, index);
    }

    //IList

    virtual public bool IsFixedSize {
      get {return false;}
    }

    virtual public bool IsReadOnly  {
      get {return false;}
    }
    
    public Address this[int index] {
      get {
        if(index<_objList.Count)
          return (Address)_objList[index];

        return null;
      }
      set {
        if(index<_objList.Count)
          _objList[index]=value;
      }
    }
    
    object IList.this[int index] {
      get {
        if(index<_objList.Count)
          return _objList[index];

        return null;
      }
      set {
        if(index<_objList.Count)
          _objList[index]=value;
      }
    }

    virtual public int Add(object value)  {
      return _objList.Add((Address)value);
    }

    virtual public void Clear() {
      _objList.Clear();
    }

    virtual public bool Contains(object value)  {
      return _objList.Contains(value);
    }

    virtual public int IndexOf(object value)  {
      return _objList.IndexOf((Address)value);
    }

    virtual public void Insert(int Index, object value) {
      _objList.Insert(Index,(Address)value);
    }

    virtual public void Remove(object value) {
      _objList.Remove((Address)value);
    }

    virtual public void RemoveAt(int index) {
      _objList.RemoveAt(index);
    }


    virtual public IEnumerator GetEnumerator() {
      return new AddressCollectionEnumerator(this);
    }
    
    /// <summary>
    /// Enumerator nested class
    /// </summary>
    class AddressCollectionEnumerator : IEnumerator  {

      int _index=-1;
      AddressCollection _collection;

      internal AddressCollectionEnumerator(AddressCollection collectionClass)  {
        _collection=collectionClass;

        if(_collection._objList.Count==0)
          _index=-1;
      }

      public bool MoveNext()  {

        if(_index<_collection._objList.Count-1) {
          _index++;
          return true;
        }
        return false;

      }

      public void Reset() {
        _index=-1;
      }

      public Object Current {
        get {
          if(_index>=0)
            return _collection._objList[_index];
          else
            throw new InvalidOperationException("Can't do that");
        }
      }
    }
  }
}
