using System;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using Wrox.Samples;

namespace Wrox.Samples.AddressControl
{
	/// <summary>
	/// Public accessor class to reference objects of type Wrox.Samples.ExtTextBox.
	/// </summary>
	public class ExtTextField: System.ComponentModel.Component
	{
		private ExtTextBox mControl;

    /// <summary>
    /// Initializes a new instance of the Wrox.Samples.AddressControl.ExtTextField class. 
    /// </summary>    
    /// <remarks>
    /// This parameterless constructor causes the ExtTextField to create and encapsulate a 
    /// new internal ExtTextBox instance which does not reference any physical control. This
    /// is useful for creating a detatched ExtTextField object that holds property values.
    /// </remarks>
    public ExtTextField()
    {
      mControl = new ExtTextBox();
    }
    
    /// <summary>
    /// Initializes a new instance of the Wrox.Samples.AddressControl.ExtTextField class, 
    /// encapsulating an ExtTextBox control.
    /// </summary>    
    /// <param name="TextBoxControl"> The ExtTextBox control to be encapsulated.</param>
    public ExtTextField(ExtTextBox TextBoxControl)
		{
      mControl = TextBoxControl;
		}

    /// <summary>
    /// Gets or sets the current text in the ExtTextBox control referenced by this ExtTextField object.
    /// </summary>
    [Category("Appearance"), Browsable(true), DefaultValue("")] 
    public string Text
    {
      get {return mControl.Text;}
      set {mControl.Text = value;}
    }

    /// <summary>
    /// Gets or sets the foreground color of the ExtTextBox control referenced by this ExtTextField object.
    /// </summary>
    [Category("Appearance"), Browsable(true), DefaultValue(typeof(Color), "WindowText")] 
    public Color ForeColor
    {
      get {return mControl.ForeColor;}
      set {mControl.ForeColor = value;}
    }

    /// <summary>
    /// Gets or sets the background color of the ExtTextBox control referenced by this ExtTextField object.
    /// </summary>
    [Category("Appearance"), Browsable(true),
    DefaultValue(typeof(Color), "Window")] 
    public Color BackColor
    {
      get {return mControl.BackColor;}
      set {mControl.BackColor = value;}
    }

//    private bool ShouldSerializeBackColor()
//    {
//      return (mControl.BackColor != SystemColors.Window);
//    }
//    
//    private void ResetBackColor()
//    {
//      mControl.BackColor = SystemColors.Window;
//    }    


    /// <summary>
    /// Gets or sets the IsRequired value of the ExtTextBox control referenced by this ExtTextField object.
    /// </summary>
    [Category("Behavior"), Browsable(true), DefaultValue(false)] 
    public bool IsRequired
    {
      get {return mControl.IsRequired;}
      set {mControl.IsRequired = value;}
    }

    /// <summary>
    /// Appends text to the current text of the ExtTextBox control referenced by this ExtTextField object.
    /// </summary>
    /// <param name="text"> The text to append to the current contents of the Wrox.Samples.ExtTextBox.</param>
    public void AppendText(string text)
    {
      mControl.AppendText(text);
    }
	}
}
